/*
 * Sone - ParserFilter.java - Copyright © 2011–2013 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.sone.template;

import static java.lang.String.valueOf;

import java.io.IOException;
import java.io.StringReader;
import java.util.Collections;
import java.util.Map;

import net.pterodactylus.sone.core.Core;
import net.pterodactylus.sone.data.Sone;
import net.pterodactylus.sone.text.Part;
import net.pterodactylus.sone.text.SoneTextParser;
import net.pterodactylus.sone.text.SoneTextParserContext;
import net.pterodactylus.sone.web.page.FreenetRequest;
import net.pterodactylus.util.template.Filter;
import net.pterodactylus.util.template.TemplateContext;

/**
 * Filter that filters a given text through a {@link SoneTextParser} and returns the parsed {@link Part}s.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class ParserFilter implements Filter {

	private final Core core;
	private final SoneTextParser soneTextParser;

	public ParserFilter(Core core, SoneTextParser soneTextParser) {
		this.core = core;
		this.soneTextParser = soneTextParser;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Object format(TemplateContext templateContext, Object data, Map<String, Object> parameters) {
		String text = valueOf(data);
		Object sone = parameters.get("sone");
		if (sone instanceof String) {
			sone = core.getSone((String) sone).orNull();
		}
		FreenetRequest request = (FreenetRequest) templateContext.get("request");
		SoneTextParserContext context = new SoneTextParserContext(request, (Sone) sone);
		try {
			return soneTextParser.parse(context, new StringReader(text));
		} catch (IOException ioe1) {
			/* no exceptions in a StringReader, ignore. */
			return Collections.<Part>emptyList();
		}
	}

}
